/***************************************************************************************************************
 * ArDAM1021 Lite edition - Control the Soekris DAM1941 by an Arduino Nano with a 128x64 OLED display
 *
 * Project Page: 
 *
 * v1.13    14/02/2019 : - DAM1941 version (adapted by euro21)
 * v0.92    03/07/2017 : - Diyaudio.com GB edition (different IR pin)
 * v0.91    31/12/2015 : - Code cleanup for release.
 * v0.90    26/08/2015 : - Code split from Universal USB to I2S Indicator project.
 *
  **************************************************************************************************************/


#include "U8glib.h"
#include <stdlib.h>
#include <IRremote.h>

// setup u8g object - select according to your specific OLED
//U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NO_ACK);                  // 0.96" OLED Display using I2C which does not send ACK
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NO_ACK);                     // 1.30" OLED Display which does not send ACK

#define NUM_FONT u8g_font_fur35n        // Font for displaying the Volume 
#define TEXT_FONT u8g_font_timB10       // Font for displaying text
#define LOGO_FONT u8g_font_fub17        // Font for displaying logo

// OWN Remote control codes. Change to match your remote's.

#define VOLUP_CODE    0xBF708F          // Code for Volume up                   I<<
#define VOLDOWN_CODE  0xBFB04F          // Code for Volume down                 >>I

#define SOURCE7_CODE  0xBF50AF          // Code for source 4 (Auto)             ON/OFF
#define SOURCE0_CODE  0xBF00FF          // Code for source 0 (USB)              INTRO
#define SOURCE1_CODE  0xBF28D7          // Code for source 1 (AES/EBU)          1
#define SOURCE2_CODE  0xBFF00F          // Code for source 2 (BNC)              2
#define SOURCE3_CODE  0xBF807F          // Code for source 3 (RCA)              3
#define SOURCE4_CODE  0xBF40BF          // Code for source 4 (Toslink OPT)      4

#define FILT1_CODE  0xBFC837            // Code for Filter 1  (Linear)          <<
#define FILT2_CODE  0xBFE817            // Code for Filter 2  (Mixed)           PAUSE
#define FILT3_CODE  0xBF08F7            // Code for Filter 3  (Minimum)         >>
#define FILT4_CODE  0xBF48B7            // Code for source 4  (Soft)            PLAY

byte x_dim;                           // Display resolution
byte y_dim;                           // Display resolution

byte str_height;
byte str_width;
byte char_width;

byte source = 7;                      // Variable to hold the Input number. 0:USB; 1:AES/EBU; 2:BNC; 3:RCA; 4:OPT; 7:auto
byte source_old = -1;                  // Variable to hold the old Input number. 

byte type = 0;                        // Variable to hold the signal type. 0 for PCM, 1 for DSD.
byte type_old = 2;                    // Variable to hold the old signal type. 0 for PCM, 1 for DSD.



float SR = 32;                     // Variable to hold the detected sampling rate (or DSD type) as float.


char stringSR[10];                   // Variable to hold the detected sampling rate (or DSD type) as string.


int Vol = 3;                         // Variable to hold the volume as int.
char stringVol[10];                  // Variable to hold the volume as string.
int Vol_display = -3;                // Variable to hold the display volume as int.

String filter1 = "Linear";           // The name of the filter F4
String filter2 = "Mixed";            // The name of the filter F5
String filter3 = "Minimum";          // The name of the filter F6
String filter4 = "Soft";             // The name of the filter F7

char filter[10];                     // Variable to hold the selected filter.
byte filt_num = 4;                   // Variable to hold the number of the selected filter.


// int Phase= 0;                       // Phase 0= normal, 1=inverted

String serialin;                     // Variable to hold the data read from the serial port.
char Control_char = 0;               // Control character (L, F, I, V, P)


unsigned long previousMillis = 0;
long source_disp_interval = 2000;


byte RECV_PIN = 4;                  // IR Receiver input pin.
IRrecv irrecv(RECV_PIN);
decode_results results;

int prev_result;

// ------------------------ Display Sampling Rate ----------------------------------------------------
void disp_SR(void) {    
  
  u8g.setFont(TEXT_FONT);
  str_height = u8g.getFontAscent()-u8g.getFontDescent(); 
  
  if(SR==0)
  {
    str_width = u8g.getStrWidth("Unlock");    
    u8g.setPrintPos((x_dim-str_width), (y_dim/2+str_height/2));   // Print to right middle
    u8g.print("Unlock");   
  }
  
  else if ((SR-(int)SR) > 0 )
  {
    str_width = u8g.getStrWidth(dtostrf(SR, 0, 1, stringSR));  
    u8g.setPrintPos((x_dim-str_width), str_height);               // Print to right upper corner 
    u8g.print(SR, 1);    
  }
  else if ((SR-(int)SR) == 0 )
  {
    str_width = u8g.getStrWidth(dtostrf(SR, 0, 1, stringSR)); 
    u8g.setPrintPos((x_dim-str_width), str_height);               // Print to right upper corner    
    u8g.print(SR, 1);    
  }
}

// ------------------------ Display Source -------------------------------------------------------------
void disp_source(void)
  {
    u8g.setFont(TEXT_FONT);
        
     if (source == 7 )
    {      
      u8g.setPrintPos(0, y_dim);                                // Print to left bottom corner
      u8g.print("Auto");
    }

    else if (source == 0 )
    {      
      u8g.setPrintPos(0, y_dim);                                 // Print to left bottom corner
      u8g.print("USB");
    }
    
    else if (source == 1 )
    {      
      u8g.setPrintPos(0, y_dim);                                  // Print to left bottom corner
      u8g.print("AES/EBU");
    }
    else if (source == 2 )
    {      
      u8g.setPrintPos(0, y_dim);                        // Print to left bottom corner
      u8g.print("BNC");
    }
    
    else if (source == 3 )
    {      
      u8g.setPrintPos(0, y_dim);                        // Print to left bottom corner   
      u8g.print("RCA");
    }
    
    else if (source == 4 )
    {      
      u8g.setPrintPos(0, y_dim);                                    // Print to left bottom corner 
      u8g.print("OPT");
    }

 }

// ------------------------ Display Signal Type ---------------------------------------------------------
void disp_type(void)
  {
    u8g.setFont(TEXT_FONT);    
    str_height = u8g.getFontAscent()-u8g.getFontDescent();
   
    if (type == 0)
    {
      u8g.setPrintPos(0, str_height);                               // Print to left upper corner
      u8g.print("PCM");
    }
    if (type == 1)
    {
      u8g.setPrintPos(0, str_height);                                // Print to left upper corner
      u8g.print("DSD");
    }

  }

// ------------------------ Display Filter -------------------------------------------------------------
void disp_filter(void)
  {    
      u8g.setFont(TEXT_FONT);          
      str_width = u8g.getStrWidth(filter);

      u8g.setPrintPos(x_dim-str_width, y_dim);                        // Print to right bottom corner
      u8g.print(filter);
  }

// ------------------------ Display Volume ------------------------------------------------------------
void disp_volume(void)
  {
    u8g.setFont(NUM_FONT);

    str_height = u8g.getFontAscent()-u8g.getFontDescent();    
    str_width = u8g.getStrWidth(itoa(Vol_display, stringVol, 10)); //only for widht
    
    if( Vol_display > 0)
    {
      u8g.setPrintPos((x_dim-str_width)/2, (y_dim/2+str_height/2));     // print to middle of the display
        Vol_display = -1*Vol ;                                          // ghost status message: V+64
    }
    else
    {
      char_width = u8g.getStrWidth("-");
      u8g.setPrintPos( ((x_dim-str_width)/2) - char_width/2  , (y_dim/2+str_height/2));     // print to middle of the display -1/2 char left
    }
    
    u8g.print(Vol_display);

  }

// ---------------------------- Read signal parameters --------------------------------------------------
void DAM_read(void) 
{

  if(Serial.available())
  {    
            
      serialin = Serial.readStringUntil('\n');
      Control_char = serialin.charAt(0);
            
    if( Control_char == 'L' )
    {       
        if(serialin == "L000\r")                          //Unlock
        {SR = 000;
        type = 0 ;                                        //  PCM        
        }
      else
      if(serialin == "L384\r")
        {SR = 384;
        type = 0 ;                                      //  PCM        
        }
        else
          if(serialin == "L352\r")
            {SR = 352.8;
            type = 0 ;                                   //  PCM            
            }
            else
              if(serialin == "L192\r")
                {SR = 192;
                type = 0 ;                               //  PCM                
                }
                else
                  if(serialin == "L176\r")
                    {SR = 176.4;
                    type = 0 ;                            //  PCM                   
                    }
                    else
                      if(serialin == "L096\r")
                        {SR = 96;
                        type = 0 ;                        //  PCM                        
                        }
                        else
                          if(serialin == "L088\r")
                            {SR = 88.2;
                            type = 0 ;                    //  PCM                            
                            }
                            else
                              if(serialin == "L048\r")
                                {SR = 48;
                                type = 0 ;                //  PCM                                
                                }
                                else
                                  if(serialin == "L044\r")
                                    {SR = 44.1;
                                    type = 0 ;                     //  PCM                                    
                                    }
                                    else
                                      if(serialin == "L02M\r")      //256
                                      {SR = 256;
                                      type = 1;                     //DSD                                      
                                      }
                                      else
                                        if(serialin == "L05M\r")    // 512
                                        {SR = 560;
                                        type = 1;                     //DSD                                        
                                        }
                                        else
                                          if(serialin == "L11M\r")  // 1120
                                          {SR = 1120;
                                          type = 1;                     //DSD                                           
                                          } 
      }                                  
    
      else if( Control_char=='I' )      
      {                                               
        if(serialin == "I7\r")                                //Auto
           { source = 7;        
            }
          else
            if(serialin == "I0\r")                            //USB
               { source = 0;                
                 }
              else
                if(serialin == "I1\r")                        //AES/EBU
                  {  source = 1;                    
                     }
                    else
                      if(serialin == "I2\r")                  //BNC
                        {  source = 2;                          
                           }
                          else
                            if(serialin == "I3\r")            //RCA
                              {  source = 3;                                
                                 }
                                else
                                  if(serialin == "I4\r")      //OPT
                                   {   source = 4;                                      
                                       }
      }
     
      else if( Control_char=='F' )  
      {        
        if(serialin == "F4\r")                                //Linear
           { filt_num = 4;
             strncpy(filter, filter1.c_str(), 10);             
            }
            else  
              if(serialin == "F5\r")                          //Mixed
                  {  filt_num = 5;
                     strncpy(filter, filter2.c_str(), 10);                     
                    }
                    else  
                        if(serialin == "F6\r")                //Minimum
                          {  filt_num = 6;
                             strncpy(filter, filter3.c_str(), 10);                            
                            }
                            else  
                                if(serialin == "F7\r")        //Soft
                                  {   filt_num = 7;
                                      strncpy(filter, filter4.c_str(), 10);                                      
                                     }
      }
      else if( Control_char=='V' )                            // Volume char
      {                                                                                                  
              serialin.toCharArray(stringVol,6);              // for example 'V-11/r'       
              sscanf(stringVol, "V%d", &Vol_display);         // first char is Control_char, Volume to integer display                
      }
      
       serialin = "";   //Flush   
 
 }  //Serial in for DAM

}
//----------------------------------------------------------------
//----------------------------------------------------------------

void setup(void) 
{
  Serial.begin(115200);  
    
  irrecv.enableIRIn();                       // Start the IR receiver

  x_dim = u8g.getWidth();
  y_dim = u8g.getHeight();
               
          previousMillis = millis();            // start display delay counting
            
  u8g.firstPage();                              // picture loop - display logo
  do {
//    draw_logo();
  }  while (u8g.nextPage());

  //--------------------------------------------------------------------------------
  //    Set DAM to -3 Volume
  //--------------------------------------------------------------------------------
     Serial.print("V-");       // output the new volume level to the DAC's serial port
     Serial.println(Vol);      // output the new volume level to the DAC's serial port 
  //--------------------------------------------------------------------------------
  delay(1000);

}

void draw_logo(void)
{
    u8g.setFont(LOGO_FONT);  
    str_height = u8g.getFontAscent()-u8g.getFontDescent();
    str_width = u8g.getStrWidth("DAM 1941");
                     
    u8g.setPrintPos( (x_dim/2-str_width/2), (y_dim/2+str_height/2));
    u8g.print("DAM 1941");
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

 void IR_read(void)                               // Read IR input
{
  if (irrecv.decode(&results)) {                  // If IR code is received, get the IR code.
        prev_result == -1;
        
      if (results.value == SOURCE0_CODE) {          // What to do if the Source 0 button is pressed      
        Serial.println("I0");        
        source = 0;        
        }
      else if (results.value == SOURCE1_CODE) {      // What to do if the Source 1 button is pressed
        Serial.println("I1");        
        source = 1;        
        }
      else if (results.value == SOURCE2_CODE)  {      // What to do if the Source 2 button is pressed        
        Serial.println("I2");        
        source = 2;        
        }
      else if (results.value == SOURCE3_CODE)  {      // What to do if the Source 3 button is pressed        
        Serial.println("I3");        
        source = 3;        
        }
      else if (results.value == SOURCE4_CODE)  {      // What to do if the Source 4 button is pressed        
        Serial.println("I4");        
        source = 4;                
        }    
      else if ((results.value == SOURCE7_CODE)  ) {      // What to do if the Source 7 button is pressed        
        Serial.println("I7");        
        source = 7;                
        }
        
      else if (results.value == FILT1_CODE)            // What to do if the Filter 1 button is pressed  
        {      
          Serial.println("F4");
          filt_num = 1;              
        } 
      else if (results.value == FILT2_CODE) 
        {
        Serial.println("F5");
        filt_num = 2;
        }             
      else if (results.value == FILT3_CODE) 
        {
        Serial.println("F6");   
        filt_num = 3;
        }  
      else if (results.value == FILT4_CODE) 
        {
        Serial.println("F7");
        filt_num = 4;
        }  
    
      else if (results.value == VOLUP_CODE)  {      // What to do if the Volume Up button is pressed
        
        if (((Vol <= 99) && (Vol > 0)) )
          {
            Vol-=1;
            Serial.print("V-");       // output the new volume level to the DAC's serial port
            Serial.println(Vol);      // output the new volume level to the DAC's serial port        
            prev_result == VOLUP_CODE;
          }
    }

    else if (results.value == VOLDOWN_CODE)  {      // What to do if the Volume Down button is pressed
        
        if (((Vol < 99) && (Vol >= 0)) )
          {
            Vol+=1;
            Serial.print("V-");       // output the new volume level to the DAC's serial port
            Serial.println(Vol);      // output the new volume level to the DAC's serial port    
            prev_result == VOLDOWN_CODE;
          }
    }

    else if (results.value == 0xFFFFFFFF) {
    
      if (results.value == VOLUP_CODE)        // What to do if the Volume Up button is pressed
      {
        if ( (Vol <= 99) && (Vol > 0) )
          {
            Vol-=1;
            Serial.print("V-");       // output the new volume level to the DAC's serial port
            Serial.println(Vol);      // output the new volume level to the DAC's serial port            
            prev_result == -1;
          }
       } 
      if (results.value == VOLDOWN_CODE)        // What to do if the Volume Down button is pressed
      {
         if ((Vol < 99) && (Vol >= 0) )
          {
            Vol+=1;
            Serial.print("V-");       // output the new volume level to the DAC's serial port
            Serial.println(Vol);      // output the new volume level to the DAC's serial port
            prev_result == -1;
          }        
      }
      
    }
  irrecv.resume();                                 // Resume decoding (necessary!)
  
  }

}
  
 
void loop()
{
      unsigned long currentMillis = millis();           
        
                      IR_read();
                      DAM_read(); 
                      
 if ((currentMillis - previousMillis) > source_disp_interval)
              {
                previousMillis = currentMillis;            
                    
                u8g.firstPage();                              // picture loop - display all
                do {
                     disp_source();
                     disp_filter();
                     disp_type();                      
                     disp_SR();                     
                     disp_volume();                      
                   }  while (u8g.nextPage());  
              }
       delay(1000);
}
